//
// impressionistUI.h
//
// The user interface part for the program. They should be subclassed and I am just lazy to.
// You may generate the similar code using FLUID, but I just hate the readability of the 
// generated code. So, I act as FLUID in this file.
//

#include <FL/fl_ask.h>

#include "impressionistUI.h"
#include "impressionistDoc.h"

//------------------------------------- Help Functions --------------------------------------------
ImpressionistUI* ImpressionistUI::whoami(Fl_Menu_* o)	// from menu item back to UI itself
{
	return ( (ImpressionistUI*)(o->parent()->user_data()) );
}

//--------------------------------- Callback Functions --------------------------------------------
void ImpressionistUI::cb_load_image(Fl_Menu_* o, void* v) 
{
	ImpressionistDoc *pDoc=whoami(o)->getDocument();

	char* newfile = fl_file_chooser("Open File?", "*.bmp", pDoc->getImageName() );
	if (newfile != NULL) {
		pDoc->loadImage(newfile);
	}
}

void ImpressionistUI::cb_save_image(Fl_Menu_* o, void* v) 
{
	ImpressionistDoc *pDoc=whoami(o)->getDocument();

	char* newfile = fl_file_chooser("Save File?", "*.bmp", "save.bmp" );
	if (newfile != NULL) {
		pDoc->saveImage(newfile);
	}
}

void ImpressionistUI::cb_brushes(Fl_Menu_* o, void* v) 
{
	whoami(o)->m_brushDialog->show();
}

void ImpressionistUI::cb_exit(Fl_Menu_* o, void* v) 
{
	whoami(o)->m_mainWindow->hide();
	whoami(o)->m_brushDialog->hide();
}

void ImpressionistUI::cb_exit2(Fl_Widget* o, void* v) 
{
	ImpressionistUI* pUI=(ImpressionistUI*)(o->user_data());
	pUI->m_mainWindow->hide();
	pUI->m_brushDialog->hide();
}

void ImpressionistUI::cb_about(Fl_Menu_* o, void* v) 
{
	fl_message("Impressionist FLTK version for CSE557, Winter 2000");
}

//------- UI should keep track of the current for all the controls for answering the query from Doc ---------
void ImpressionistUI::cb_sizeSlides(Fl_Widget* o, void* v)
{
	((ImpressionistUI*)(o->user_data()))->m_nSize=int( ((Fl_Slider *)o)->value() ) ;
}

void ImpressionistUI::cb_brushChoice(Fl_Widget* o, void* v)
{
	ImpressionistDoc *pDoc=((ImpressionistUI *)(o->user_data()))->getDocument();

	pDoc->setBrushType((int)v);
}

//---------------------------------- per instance functions --------------------------------------
ImpressionistDoc* ImpressionistUI::getDocument()
{
	return m_pDoc;
}

void ImpressionistUI::show() {
	m_mainWindow->show();
	m_paintView->show();
	m_origView->show();
}

void ImpressionistUI::resize_windows(int w, int h) {
	m_paintView->size(w,h);
	m_origView->size(w,h);
}

void ImpressionistUI::setDocument(ImpressionistDoc* doc)
{
	m_pDoc = doc;

	m_origView->m_pDoc = doc;
	m_paintView->m_pDoc = doc;
}

int ImpressionistUI::getSize()
{
	return m_nSize;
}

// menu definition
Fl_Menu_Item ImpressionistUI::menuitems[] = {
	{ "&File",		0, 0, 0, FL_SUBMENU },
		{ "&Load Image...",	FL_ALT + 'l', (Fl_Callback *)ImpressionistUI::cb_load_image },
		{ "&Save Image...",	FL_ALT + 's', (Fl_Callback *)ImpressionistUI::cb_save_image },
		{ "&Brushes...",	FL_ALT + 'b', (Fl_Callback *)ImpressionistUI::cb_brushes, 0, FL_MENU_DIVIDER },
		{ "&Exit",			FL_ALT + 'e', (Fl_Callback *)ImpressionistUI::cb_exit },
		{ 0 },

	{ "&Help",		0, 0, 0, FL_SUBMENU },
		{ "&About",	FL_ALT + 'a', (Fl_Callback *)ImpressionistUI::cb_about },
		{ 0 },

	{ 0 }
};

Fl_Menu_Item ImpressionistUI::brushTypeMenu[NUM_BRUSH_TYPE+1] = {
  {"Points",			FL_ALT+'p', (Fl_Callback *)ImpressionistUI::cb_brushChoice, (void *)BRUSH_POINTS},
  {"Lines",				FL_ALT+'l', (Fl_Callback *)ImpressionistUI::cb_brushChoice, (void *)BRUSH_LINES},
  {"Circles",			FL_ALT+'c', (Fl_Callback *)ImpressionistUI::cb_brushChoice, (void *)BRUSH_CIRCLES},
  {"Scattered Points",	FL_ALT+'q', (Fl_Callback *)ImpressionistUI::cb_brushChoice, (void *)BRUSH_SCATTERED_POINTS},
  {"Scattered Lines",	FL_ALT+'m', (Fl_Callback *)ImpressionistUI::cb_brushChoice, (void *)BRUSH_SCATTERED_LINES},
  {"Scattered Circles",	FL_ALT+'d', (Fl_Callback *)ImpressionistUI::cb_brushChoice, (void *)BRUSH_SCATTERED_CIRCLES},
  {0}
};

ImpressionistUI::ImpressionistUI() {
	m_mainWindow = new Fl_Window(600, 300, "Impressionist");
		m_mainWindow->user_data((void*)(this));	// record self to be used by static callback functions
		// install menu bar
		m_menubar = new Fl_Menu_Bar(0, 0, 600, 25);
		m_menubar->menu(menuitems);

		Fl_Group* group = new Fl_Group(0, 25, 600, 275);

		// install paint view window
		m_paintView = new PaintView(0, 25, 300, 275, "This is the paint view");
        m_paintView->box(FL_DOWN_FRAME);

		// install original view window
		m_origView = new OriginalView(300, 25, 300, 275, "This is the orig view");
        m_origView->box(FL_DOWN_FRAME);
        m_origView->deactivate();

		group->end();
		Fl_Group::current()->resizable(group);

		m_mainWindow->callback(cb_exit2);	
		m_mainWindow->when(FL_HIDE);
    m_mainWindow->end();

	// init values
	m_nSize = 10;

	// brush dialog
	m_brushDialog = new Fl_Window(240, 80, "Brush Dialog");
		// install slider size
		m_BrushSizeSlider = new Fl_Value_Slider(10, 45, 180, 20, "Size");
		m_BrushSizeSlider->user_data((void*)(this));	// record self to be used by static callback functions
		m_BrushSizeSlider->type(FL_HOR_NICE_SLIDER);
        m_BrushSizeSlider->labelfont(FL_COURIER);
        m_BrushSizeSlider->labelsize(12);
		m_BrushSizeSlider->minimum(0);
		m_BrushSizeSlider->maximum(20);
		m_BrushSizeSlider->step(1);
		m_BrushSizeSlider->value(m_nSize);
		m_BrushSizeSlider->align(FL_ALIGN_RIGHT);
		m_BrushSizeSlider->callback(cb_sizeSlides);

		m_BrushTypeChoice = new Fl_Choice(50,10,150,25,"&Brush");
		m_BrushTypeChoice->user_data((void*)(this));	// record self to be used by static callback functions
		m_BrushTypeChoice->menu(brushTypeMenu);
		m_BrushTypeChoice->callback(cb_brushChoice);
    m_brushDialog->end();	
}
